# Pankaj Mehta's code for Bayesian Ising Approximation

# This code implements the Bayesian Ising Approximation (BIA) for linear regression 
# Should easily generalize to other linear models


############################################################################

# USER INPUT


# comment: first set working directory (fill in appropriate name)
# setwd(" the directory where this file is saved ")

# comment: then run this code
# source("mRNA_ANOVA.R")


############################################################################

# Load packages

# Generate synthetic data

#Number of data points
n_s=1000;

#Number of features

p_s=100

q_s=5;

#Generate Artificial Data with only q real elements
X <-matrix( rnorm(n_s*p_s,mean=0,sd=0.2), n_s, p_s) 
noise<-rnorm(n_s, mean=0, sd=1)

# Choose first q coefficients to be one
beta <-rep(0, p_s)
beta[1:q_s]<-1

# Generate y data 
y= X%*%beta+noise

# This generates all of the synthetic data 

# Now write code to implement the BIA

require("ggplot2")
require("reshape2")
# Find number of data points
n <-dim(X)[[1]]
# Find number of features
p<-dim(X)[[2]]

#Standerdize the data

scaled.y <-scale(y)
scaled.X<-scale(X)

# Calculate correlation matrix

Xcorrelations <-cor(scaled.X)
YXcorrelations<-cor(scaled.y, scaled.X)


#Calculate average correlation parameter
r<-sqrt((sum(Xcorrelations*Xcorrelations)-p)/(p*(p-1)))
#Calculate threshold lambda and epsilon
lambdath <-n*(1+p*r)
epsilonth <-1/lambdath



#Calculate magnetizations
depsilon=0.05*epsilonth

#Set up magnetization vector
m=matrix(rep(0, p*61), nrow=p, ncol=61)


#intialize epsilon
for (s in 1:60){
  epsilon=s*depsilon 
  #Calculate J using formula
  J<-epsilon*n*(  (Xcorrelations*Xcorrelations)/n - (t(YXcorrelations)%*%(YXcorrelations))*Xcorrelations
                +0.5*t(YXcorrelations^2)%*%(YXcorrelations^2))
  #Create J with zero diagonal
  J0<-J
  diag(J0)<-0
  #Calculate h using formula

  h<-YXcorrelations^2 -1/n+rowSums(J)

  m[,s+1]<-tanh((epsilon)*n^2/4*(h+0.5*t(J0 %*% m[,s])))
}



#Plot Resulting Graph
library (ggplot2)
library(ggplot2)
library(reshape2)
epsilonpath<-depsilon*(0:60)
df <-data.frame(epsilonpath,t(m))

opts=c(rep("red",q_s),rep("black", p-q_s)) 
# melt the data to a long format
df2 <- melt(data = df, id.vars = "epsilonpath")
# plot, using the aesthetics argument 'colour'
ggplot(data = df2, aes(x = epsilonpath, y = value, colour = variable)) + geom_line()+
  scale_colour_manual(values = opts)

